#!/usr/bin/env python3

from urllib.parse import urlparse, parse_qs

from bella import write_json_to_file


def get_playlist_title(api, pid: str):
    print(f'get playlist info {pid}')
    pl = api.get_playlist_by_id(playlist_id=pid)
    if len(pl.items) == 0:
        return False
    return pl.items[0].snippet.title


def get_video_from_playlist(api, pid: str):
    print(f'get items from playlist {pid}')
    result = api.get_playlist_items(
        playlist_id=pid,
        count=None
    )
    videos = []

    for item in result.items:
        video_id = item.snippet.resourceId.videoId
        print(f'loading video {video_id}')
        video = api.get_video_by_id(video_id=video_id, return_json=True)
        snippet = video['items'][0]['snippet']
        videos.append(dict(
            video_id=video_id,
            title=snippet['title'],
            published_at=snippet['publishedAt']
        ))
        print(f'loaded {len(videos)} items')

    return videos


def get_playlist_id(param):
    playlist_id = None if param.startswith('https://') else param
    if not playlist_id:
        query = urlparse(param).query
        queryDict = parse_qs(query)
        if 'list' in queryDict and len(queryDict['list']) > 0:
            playlist_id = queryDict['list'][0]
    return playlist_id


def get_playlist_index(api, store_dir: str, pid: str, filename: str):
    pl_title = get_playlist_title(api, pid)
    if not pl_title:
        return f'Playlist {pid} is empty or unavailable to load'
    videos = get_video_from_playlist(api, pid)
    if videos and len(videos) > 0:
        fname = filename if filename else pl_title
        fpath = f'{store_dir}/{fname}.json'
        write_json_to_file(fpath, videos)
        return f'Indexed {len(videos)} videos from playlist {pid}'
    else:
        return f'There is no video from playlist {pid}'



#!/usr/bin/env python3

import os
from urllib.parse import urlparse, parse_qs

from pytube import YouTube
from bella import read_json_from_file

from .helper import byte_to_text, find_video_index, get_title


def get_video_id(param):
    video_id = None if param.startswith('https://') else param
    if not video_id:
        query = urlparse(param).query
        queryDict = parse_qs(query)
        if 'v' in queryDict and len(queryDict['v']) > 0:
            video_id = queryDict['v'][0]
    return video_id


def get_video_by_id(api, store_dir: str, vid: str, fname: str = None):
    video_dir = f'{store_dir}/video'
    if not os.path.exists(video_dir):
        os.system(f'mkdir -p {video_dir}')

    print('Get video info', vid)
    ytUrl = f'https://youtu.be/{vid}'
    yt = YouTube(ytUrl)

    title = yt.title
    print('Loaded video file', title)
    file_name = fname if fname else yt.title

    print('Available stream:')
    for s in yt.streams:
        print(s)

    print('/' * 70)

    stream = yt.streams.get_highest_resolution()
    print('Selected stream:')
    print(stream)
    print(f'File size: {byte_to_text(stream.filesize)}')

    def on_complete(fpath):
        print(f'Finished downloading: {fpath}')

    print(f'Downloading {ytUrl}')
    stream.on_complete = on_complete
    stream.download(
        video_dir,
        file_name,
    )


def get_videos_from_indexed_list(
        api, store_dir, indexed_file, since, limit, prefix_name, prefix_num):
    print(indexed_file, since, limit, prefix_name)
    videos = read_json_from_file(indexed_file)
    total = len(videos)
    if total == 0:
        return 'List is empty!'
    begin_at = 0 if not since else find_video_index(videos, since)
    end_at = total if not limit else min(total, limit + begin_at)
    extracted = videos[begin_at:end_at]
    k = prefix_num if prefix_num else begin_at + 1
    for item in extracted:
        print('*' * 80)
        title = item['title']
        item['title'] = get_title(title, k, prefix_name, total)
        print(f'Start downloading {item["title"]}')
        get_video_by_id(api, store_dir, item['video_id'], item['title'])
        k += 1



#!/usr/bin/env python3

def find_video_index(videos: list, vid: str):
    for k, video in enumerate(videos):
        if video['video_id'] == vid:
            return k
    return 0


def get_title(vtitle: str, curr_index: int, prefix_name: str, total: int):
    if not prefix_name:
        return vtitle
    txtnum = str(curr_index).rjust(len(str(total)), '0')
    return f'{prefix_name} - {txtnum}'


def byte_to_text(bytesize: int, precision: int = 2):
    abbrevs = (
        (1 << 50, 'PB'),
        (1 << 40, 'TB'),
        (1 << 30, 'G'),
        (1 << 20, 'M'),
        (1 << 10, 'K'),
        (1, 'bytes')
    )
    if bytesize == 1:
        return '1 byte'
    for factor, suffix in abbrevs:
        if bytesize >= factor:
            break
    if factor == 1:
        precision = 0
    result = bytesize / float(factor)
    if result <= 0:
        return 0
    return '%.*f %s' % (precision, result, suffix)



#!/usr/bin/env python3

import os

from pytube import YouTube
from bella import read_json_from_file

from .helper import byte_to_text, find_video_index, get_title


def get_audio_by_id(api, store_dir: str, vid: str, fname: str = None):
    audio_dir = f'{store_dir}/audio'
    dl_dir = f'{store_dir}/downloads'
    if not os.path.exists(audio_dir):
        os.system(f'mkdir -p {audio_dir}')
    if not os.path.exists(dl_dir):
        os.system(f'mkdir -p {dl_dir}')

    print('Get video info', vid)
    ytUrl = f'https://youtu.be/{vid}'
    yt = YouTube(ytUrl)

    title = yt.title
    print('Loaded video file', title)
    file_name = fname if fname else yt.title

    print('Available stream:')
    for s in yt.streams:
        print(s)

    print('/' * 70)

    stream = yt.streams.filter(only_audio=True).order_by('abr').last()
    print('Selected stream:')
    print(stream)
    print(f'File size: {byte_to_text(stream.filesize)}')

    def on_complete(fpath):
        print('Convert to mp3', fpath)
        mp3_file = f'{audio_dir}/{file_name}.mp3'
        cmd = f'ffmpeg -i "{fpath}" -y "{mp3_file}"'
        print(f'Run command: {cmd}')
        os.system(cmd)
        print(f'Finish downloading {file_name}, mp3 saved at {mp3_file}')

    print(f'Downloading {ytUrl}')
    stream.on_complete = on_complete
    stream.download(
        dl_dir,
        file_name,
    )


def get_audios_from_indexed_list(
        api, store_dir, indexed_file, since, limit, prefix_name, prefix_num):
    print(indexed_file, since, limit, prefix_name)
    videos = read_json_from_file(indexed_file)
    total = len(videos)
    if total == 0:
        return 'List is empty!'
    begin_at = 0 if not since else find_video_index(videos, since)
    end_at = total if not limit else min(total, limit + begin_at)
    extracted = videos[begin_at:end_at]
    k = prefix_num if prefix_num else begin_at + 1
    for item in extracted:
        print('*' * 80)
        title = item['title']
        item['title'] = get_title(title, k, prefix_name, total)
        print(f'Start downloading {item["title"]}')
        get_audio_by_id(api, store_dir, item['video_id'], item['title'])
        k += 1



#!/usr/bin/env python3

import os
import getpass
from typing import Any

from bella import write_json_to_file, read_json_from_file

ENV = os.getenv('ENV', 'prod')

USER_DIR = f'/home/{getpass.getuser()}' if ENV == 'prod' else './ptester'
YTDL_CONF_DIR = f'{USER_DIR}/.config/ytdl'
CONFIG_FILE = f'{YTDL_CONF_DIR}/config.json'
YTDL_DEFAULT_STORE_DIR = f'{USER_DIR}/ytdl_files'


def load_config():
    if not os.path.exists(YTDL_CONF_DIR):
        os.system(f'mkdir -p {YTDL_CONF_DIR}')
    if not os.path.exists(CONFIG_FILE):
        if not os.path.exists(YTDL_DEFAULT_STORE_DIR):
            os.system(f'mkdir -p {YTDL_DEFAULT_STORE_DIR}')
        write_json_to_file(CONFIG_FILE, {
            'api_key': '',
            'store_dir': YTDL_DEFAULT_STORE_DIR
        })

    return read_json_from_file(CONFIG_FILE)


def get_config(key: str = None):
    c = load_config()
    if not key:
        print(c)
    elif key not in c:
        print(f'`{key}` is a invalid key')
    else:
        print(f'{key} --> {c[key] if key in c else "None"}')


def set_config(key: str, val: Any):
    c = load_config()
    c[key] = val
    write_json_to_file(CONFIG_FILE, c)
    print(f'`config.{key}` has been set to `{val}`')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ytdl', 'ytdl.utils']

package_data = \
{'': ['*']}

modules = \
['ytdl_entrypoint']
install_requires = \
['bella>=2.1.3,<3.0.0',
 'fire>=0.3.1,<0.4.0',
 'python-youtube>=0.6.0,<0.7.0',
 'pytube3>=9.6.4,<10.0.0']

entry_points = \
{'console_scripts': ['ytdl = ytdl_entrypoint:init']}

setup_kwargs = {
    'name': 'ytdl',
    'version': '1.0.0rc3',
    'description': '',
    'long_description': '# ytdl\n\nYet another CLI based YouTube downloader tool for linux.\n\n\n[![PyPI version](https://badge.fury.io/py/ytdl.svg)](https://badge.fury.io/py/ytdl)\n[![Build Status](https://travis-ci.org/ndaidong/ytdl.svg?branch=master)](https://travis-ci.org/ndaidong/ytdl)\n[![Coverage Status](https://coveralls.io/repos/github/ndaidong/ytdl/badge.svg?branch=master)](https://coveralls.io/github/ndaidong/ytdl?branch=master)\n\n\n### Features:\n\n- Download only neccessary stream, not the whole video\n- Download all videos or audios belong to a playlist\n- Download with custom name and bounding\n\nJust gather your favourite videos into a playlist, then let\'s `ytdl` download them overnight.\n\n\n## Prerequisites\n\n- Fedora 32+, Ubuntu 20+, Debian 10+\n- [python](https://www.python.org/) 3.6.5 or newer\n- [ffmpeg](https://ffmpeg.org/) 4.0 or newer\n- [YouTube API key](https://developers.google.com/youtube/registering_an_application)\n\n\n## Install\n\n\nRecommend to use [pipx](https://pipxproject.github.io/pipx/):\n\n```bash\npipx install ytdl\nytdl config\n```\n\nHowever, `pip` may work too:\n\n\n```bash\npip install ytdl\n\n# don\'t\nsudo pip install ytdl\n```\n\nBuild from source requires [poetry](https://python-poetry.org/):\n\n```bash\ngit clone git@github.com:ndaidong/ytdl.git && cd ytdl\npoetry install\n\n# use raw script\npoetry run python main.py info\npoetry run python main.py [command] [arguments]\n\n# build wheel to `./dist` folder\npoetry build\n\n# then install it\npipx install dist/ytdl-VERSION-py3-none-any.whl\n\n# test it\nytdl info\n```\n\n\n## CLIs\n\n### Basic commands\n\n| Command | Description | Shortcut |\n|--|--|--|\n| `ytdl config KEY VALUE` | Set config value | `c` |\n| `ytdl config KEY` | Show config property |\n| `ytdl playlist PLAYLIST_URL` | Get playlist index | `p` |\n| `ytdl video VIDEO_URL` | Download a video | `v` |\n| `ytdl audio VIDEO_URL` | Download audio only | `a` |\n\nAlmost cases, `VIDEO_URL` and `PLAYLIST_URL` can be replaced with video ID or playlist ID.\n\n\n### Advanced usage\n\n\n#### Config\n\nThere are 2 properties to configure: `api_key` and `store_dir`.\nAt the first time, `api_key` is empty and you have to set it before using other features.\n\n```bash\n# set new `api_key`\nytdl config api_key YOUR_OWN_YOUTUBE_API_KEY\n\n# change `store_dir` to new path\nytdl config store_dir /storage/downloads/youtube\n\n# get the current value of `api_key`\nytdl config api_key\n\n# show all\nytdl config\n```\n\nBy default, `store_dir` is being set to `/home/{YOUR_USER_NAME}/ytdl_files`, you should change it to more appropriate place.\n\n\n#### Playlist\n\nNote that this command does not download actual video/audio, but a list of indexed items.\n\n```bash\n# get playlist metadata into `{store_dir}/{title}.json`\n# this file contains a list of videos with their ID and title to download later\nytdl playlist https://www.youtube.com/playlist?list=PLAYLIST_ID\n\n# get playlist metadata into `{store_dir}/my_custom_playlist_name.json`\nytdl playlist https://www.youtube.com/playlist?list=PLAYLIST_ID my_custom_playlist_name\n```\n\nFor example if we download the playlist [Linux Tips and Tricks](https://www.youtube.com/playlist?list=PLSmXPSsgkZLsw-vEwve1O7w-Row9TIVqi)\n\nThe indexed file looks like below:\n\n![](https://imgshare.io/images/2020/08/31/playlist-indexed.png)\n\nThen we will have some powerful ways to download the videos in this list with `ytdl video` or `ytdl audio`.\n\n\n#### Video\n\nDownload a single video file.\n\n```bash\n# download a video file to `{store_dir}/video/{VIDEO_TITLE}.mp4`\nytdl video https://www.youtube.com/watch?v=VIDEO_ID\n\n# custom name\nytdl video https://www.youtube.com/watch?v=VIDEO_ID my_custom_video_name\n\n```\n\nTo download multi items from indexed playlist, please refer the following arguments:\n\n- `--index_file`: path to playlist index file (required)\n- `--since`: video ID of the video where you want to start downloading from\n- `--limit`: number of items to download, count from `since` or the begining of the list\n- `--prefix_name`: to auto naming downloaded file\n- `--prefix_num`: to auto naming downloaded file\n\nExamples:\n\n```bash\n# download all videos from saved playlist index file above\n# these video files will be stored in `{store_dir}/video`\nytdl video --index_file "/path/to/Linux Tips and Tricks.json"\n\n# download 5 videos from saved playlist index file above, since first item\nytdl video --index_file "/path/to/Linux Tips and Tricks.json" --limit 5\n\n# download 5 videos from saved playlist index file above, with prefix name\nytdl video --index_file "/path/to/Linux Tips and Tricks.json" --limit 5 --prefix_name "Linux Tutorial"\n# downloaded videos should look like "Linux Tutorial - 1.mp4", "Linux Tutorial - 2.mp4" and so on\n# prefix_name will be useful when you want to put these files into an already created list for your different purpose\n\n# download 5 videos from saved playlist index file above, with prefix name and prefix number\nytdl video --index_file "/path/to/Linux Tips and Tricks.json" --limit 5 --prefix_name "Linux Tutorial" --prefix_num 25\n# this will be useful for the playlists those are splited to multi parts\n# in this case, your serie "Linux Tutorial" had already 24 items before, now count from 25 onwards\n# downloaded videos should look like "Linux Tutorial - 25.mp4", "Linux Tutorial - 26.mp4" and so on\n\n# similar to above command, but start from given item\nytdl video --index_file "/path/to/Linux Tips and Tricks.json" --since VIDEO_ID --limit 5 --prefix_name "Linux Tutorial" --prefix_num 25\n```\n\nWhile downloading video, the stream with highest `resolution` will be selected.\n\n\n#### Audio\n\nThis is similar to `ytdl video`, but only download audio file.\nWhile downloading, the stream with highest `abr` (average bitrate) will be selected.\n\n\n\n```bash\n# download a audio file to `{store_dir}/audio/{VIDEO_TITLE}.mp3`\nytdl audio https://www.youtube.com/watch?v=VIDEO_ID\n\n# custom name\nytdl audio https://www.youtube.com/watch?v=VIDEO_ID my_custom_audio_name\n```\n\nTo download multi items from indexed playlist, please refer the following arguments:\n\n- `--index_file`: path to playlist index file (required)\n- `--since`: video ID of the video where you want to start downloading from\n- `--limit`: number of items to download, count from `since` or the begining of the list\n- `--prefix_name`: to auto naming downloaded file\n- `--prefix_num`: to auto naming downloaded file\n\n\nExamples:\n\n```bash\n# download all audios from saved playlist index file above\n# these audio files will be stored in `{store_dir}/audio`\nytdl audio --index_file "/path/to/Linux Tips and Tricks.json"\n\n# download 5 audios from saved playlist index file above, since first item\nytdl audio --index_file "/path/to/Linux Tips and Tricks.json" --limit 5\n\n# download 5 audios from saved playlist index file above, with prefix name\nytdl audio --index_file "/path/to/Linux Tips and Tricks.json" --limit 5 --prefix_name "Linux Tutorial"\n# downloaded audios should look like "Linux Tutorial - 1.mp3", "Linux Tutorial - 2.mp3" and so on\n# prefix_name will be useful when you want to put these files into an already created list for your different purpose\n\n# download 5 audios from saved playlist index file above, with prefix name and prefix number\nytdl audio --index_file "/path/to/Linux Tips and Tricks.json" --limit 5 --prefix_name "Linux Tutorial" --prefix_num 25\n# this will be useful for the playlists those are splited to multi parts\n# in this case, your serie "Linux Tutorial" had already 24 items before, now count from 25 onwards\n# downloaded audios should look like "Linux Tutorial - 25.mp3", "Linux Tutorial - 26.mp3" and so on\n\n# similar to above command, but start from given item\nytdl audio --index_file "/path/to/Linux Tips and Tricks.json" --since VIDEO_ID --limit 5 --prefix_name "Linux Tutorial" --prefix_num 25\n```\n\nDownloaded stream will be convert to .mp3 with `ffmpeg`.\n\n\n## Dependencies\n\nThis lib was built on top of the following packages:\n\n| Dependency | License |\n|--|--|\n| [pytube3](https://github.com/get-pytube/pytube3) | MIT |\n| [python-youtube](https://github.com/sns-sdks/python-youtube) | MIT |\n| [python-fire](https://github.com/google/python-fire) | Apache License v2 |\n\n\n## Test\n\n```bash\ngit clone git@github.com:ndaidong/ytdl.git && cd ytdl\npoetry install\n./run_test.sh\n```\n\n\n# License\n\nThe MIT License (MIT)\n',
    'author': 'Dong Nguyen',
    'author_email': 'ndaidong@gmail.com',
    'maintainer': 'Dong Nguyen',
    'maintainer_email': 'ndaidong@gmail.com',
    'url': 'https://pypi.org/project/ytdl',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3

import os

import fire
from pyyoutube import Api

from .utils.config import load_config, get_config, set_config
from .utils.playlist import get_playlist_id, get_playlist_index
from .utils.video import get_videos_from_indexed_list, \
    get_video_id, get_video_by_id
from .utils.audio import get_audios_from_indexed_list, get_audio_by_id


conf = load_config()

store_dir = os.path.normpath(
    os.path.join(os.getcwd(), conf['store_dir'])
) if not os.path.isabs(conf['store_dir']) else conf['store_dir']


def show_info():
    print('*' * 72)
    print(f'YouTube API key: `{conf["api_key"]}`')
    print(f'Actual store_dir: `{store_dir}`')
    print('*' * 72)


def configure(key=None, val='---'):
    if val == '---':
        return get_config(key) if key else show_info()
    else:
        return set_config(key, val)


def get_playlist(url=None, filename=None):
    pid = get_playlist_id(url)
    if not pid:
        return 'Please provide playlist ID or full URL'
    api = Api(api_key=conf['api_key'])
    return get_playlist_index(api, store_dir, pid, filename)


def get_video(
    url=None,
    file_name=None,
    index_file=None,
    since=None,
    limit=None,
    prefix_name=None,
    prefix_num=None
):
    api = Api(api_key=conf['api_key'])
    if index_file and os.path.exists(index_file):
        return get_videos_from_indexed_list(
            api, store_dir,
            index_file, since, limit, prefix_name, prefix_num
        )
    vid = get_video_id(url)
    if not vid:
        return 'Please provide video ID or full URL'
    return get_video_by_id(api, store_dir, vid, file_name)


def get_audio(
    url=None,
    file_name=None,
    index_file=None,
    since=None,
    limit=None,
    prefix_name=None,
    prefix_num=None
):
    api = Api(api_key=conf['api_key'])
    if index_file and os.path.exists(index_file):
        return get_audios_from_indexed_list(
            api, store_dir,
            index_file, since, limit, prefix_name, prefix_num
        )
    vid = get_video_id(url)
    if not vid:
        return 'Please provide video ID or full URL'
    return get_audio_by_id(api, store_dir, vid, file_name)


def init():
    if 'api_key' not in conf:
        print('Please set `api_key` first. Command help:')
        print('*' * 72)
        print('  ytdl config api_key YOUR_API_KEY')
        print('*' * 72)
        return False

    return fire.Fire({
        'config': configure,
        'c': configure,
        'playlist': get_playlist,
        'p': get_playlist,
        'video': get_video,
        'v': get_video,
        'audio': get_audio,
        'a': get_audio,
        'info': show_info
    })


if __name__ == '__main__':
    init()
